/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query;


import java.util.Collection;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Criterion;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Order;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Restrictions;


/**
 * <tt>Criteria</tt> is a simplified API for retrieving JCR Nodes by composing <tt>Criterion</tt> objects. This is a
 * very convenient approach for functionality like "search" screens where there is a variable number of conditions to be
 * placed upon the result set.<br>
 * The <tt>JCRCriteriaFactory</tt> is a factory for <tt>Criteria</tt>. <tt>Criterion</tt> instances are usually obtained
 * via the factory methods on <tt>Restrictions</tt>. eg.
 * 
 * <pre>
 * Calendar begin = Calendar.getInstance();
 * begin.set(2004, 0, 1);
 * Calendar end = Calendar.getInstance();
 * end.set(2008, 11, 1);
 *   	
 * Collection pets = JCRCriteriaFactory.createMgnlCriteria(&quot;//dogs//*&quot;, MgnlContext.getQueryManager(&quot;website&quot;), &quot;nt:base&quot;).add(
 * 		Restrictions.contains(&quot;@name&quot;, &quot;Nana&quot;)).add(
 * 		Restrictions.gt(&quot;@weight&quot;, new Float(10))).add(
 * 		Restrictions.between(&quot;@birthDate&quot;, begin, end).addOrder(
 * 		Order.desc(&quot;@jcr:score()&quot;)).list();
 * </pre>
 * 
 * will be translated into the following xpath statement
 * 
 * <pre>
 *  //dogs//*[((jcr:contains(@name, 'Nana')) and (@weight&gt;10.0) and (@birthDate &gt;=xs:dateTime('2004-01-01T00:00:00.000+00:00') and @birthDate &lt;=xs:dateTime('2008-12-01T23:59:59.000+00:00')))] order by @jcr:score() descending
 * </pre>
 * 
 * This API is blatantly inspired by and respectfully borrows code from Hibernate's Criteria API. <br>
 * <br>
 * @see JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String)
 * @see JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String, Class)
 * @see JCRCriteriaFactory#createMgnlCriteriaWithLimit(String, info.magnolia.cms.core.search.QueryManager, String)
 * @see JCRCriteriaFactory#createMgnlCriteriaWithLimit(String, info.magnolia.cms.core.search.QueryManager, String, Class)
 * @see Restrictions
 * @see Order
 * @author Federico Grilli
 * @version $Id: Criteria.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public interface Criteria
{

    /**
     * Add a {@link Criterion restriction} to constrain the results to be retrieved.
     * @param criterion The {@link Criterion criterion} object representing the restriction to be applied.
     * @return this (for method chaining)
     */
    public Criteria add(Criterion criterion);

    /**
     * Add an {@link Order ordering} to the result set. Only <strong>one Order criterion per query</strong> can be
     * applied. Any Order added after the first one will be ignored.
     * @param order The {@link Order order} object representing an ordering to be applied to the results.
     * @return this (for method chaining)
     */
    public Criteria addOrder(Order order);

    /**
     * Set a limit upon the number of objects to be retrieved.
     * @param maxResults the maximum number of results
     * @return this (for method chaining)
     */
    public Criteria setMaxResults(int maxResults);

    /**
     * Set the first result to be retrieved.
     * @param firstResult the first result to retrieve, numbered from <tt>0</tt>
     * @return this (for method chaining)
     */
    public Criteria setFirstResult(int firstResult);

    /*
     * Set a timeout for the underlying query.
     * @param timeout The timeout value to apply.
     * @return this (for method chaining) FIXME is it possible in JCR?
     *
      public Criteria setTimeout(int timeout);
    */

    /**
     * Get the results. <strong>The implementation should guarantee the caller that the returned Collection is never
     * null</strong>
     * @return The Collection of matched query results.
     */
    public Collection< ? > list() throws JCRQueryException;
}